#!/usr/bin/perl -w

package plain;

=head1 NAME

plain - pragma to disable magic open by giving you a 3-arg open

=head1 SYNOPSIS

    use plain "open";
    open(F, "<", "foo|") or die;	# opens file "foo|" for reading
    $name = " foo";
    open(F, "<", "$name") or die;	# opens file " foo" for writing

=head1 DESCRIPTION

Perl comes with two ways of opening a file: open() and sysopen().
open() mixes the file mode (how the file is to be opened) in with the
file name.  The rules for combining the mode and the name make it hard
to open files with spaces at the start or end of their names without
doing jiggerypokery like:

    $name =~ s{^(\s)}{./$1};
    open(FH, "< $name\0") or die;

This is neither convenient nor obvious.  This module provides you with
an open() function that can take either two arguments or three.  If
called with two arguments, it behaves exactly like Perl's regular
open().  If called with three arguments, it interprets the first as a
filehandle, the second as a open()-style mode string, and the third as
the file name.

Perl's built-in open() allows you to open pipes to processes by
specifying filenames like "|tocmd" or "fromcmd|".  The three-argument
open has no way of specifying these things.  This is construed by some
as a bug, and by others as a feature.  The special filename "-" is
grandfathered to mean STDIN or STDOUT.

=head1 AUTHOR

Nathan Torkington, C<gnat@frii.com>.  1998-12-01.

=head1 BUGS

It takes away some of the magic in open().  Sorry, Tom.  It also uses
sysopen() to do the three-arg Perl open().  This may (although it's
yet to be shown that it happens) cause problems on systems where
fopen(3) adds non-POSIX O_* constants to the open(2) call.  If this
ever gets moved into the core, we'd be able to use fopen(3) there and
bypass this problem.

=cut

use Exporter;
use File::Spec;
use Fcntl;
use strict;

use vars qw(@ISA @EXPORT_OK $VERSION);

@ISA = qw(Exporter);
@EXPORT_OK = qw(open);

$VERSION = "1.00";

my %equivalent = (
		  '<'   => O_RDONLY,
		  '>'   => O_WRONLY|O_TRUNC |O_CREAT,
		  '>>'  => O_WRONLY|O_APPEND|O_CREAT,
		  '+<'  => O_RDWR,
		  '+>'  => O_RDWR  |O_TRUNC |O_CREAT,
		  '+>>' => O_RDWR  |O_APPEND|O_CREAT,
		  );

sub open (*@) {
    # allow "real" open
    croak("one-argument open is dead.") if @_ == 1;
    return open($_[0], $_[1]) if @_ == 2;
    die 'usage: open(FH, "<", $filename)' unless @_ == 3;

    my ($fh,$mode,$name) = @_;
    exists($equivalent{$mode})
	or croak("Unknown mode: $mode\n");
    return open($fh, "$mode $name") if $name eq "-";
    return sysopen($fh, $name, $equivalent{$mode});
}

1;
